@@ -1,7 +1,8 @@
 __all__ = [
     "ds", "ds_pd",
     "atr", "atr_pd",
-    "bb",
+    "bb", "bb_pd",
+    "dmi", "dmi_pd",
     "bbw",
     "change",
     "cross",
@@ -9,7 +10,6 @@ __all__ = [
     "crossunder",
     "cum",
     "dd",
-    "dmi",
     "donchian",
     "ema",
     "highest",
@@ -24,5 +24,5 @@ __all__ = [
 ]
 from .alan_ta import (
     ds, atr, bb, bbw, change, cross, crossover, crossunder, cum, dd, dmi, donchian, ema, highest, hma, lowest, pivothigh, pivotlow, rma, sma, stdev, supertrend, tr, wma, wpr,  rsi, 
-    atr_pd, ds_pd
+    atr_pd, ds_pd, bb_pd, dmi_pd
 )
\ No newline at end of file
