@@ -1,21 +1,20 @@
 import numpy as np
 import numba as nb
 import pandas as pd
-import time
 
 
 def pandas_decorator(func):
     def wrapper(df_ohlc: pd.DataFrame, *args, **kwargs):
+        print(f"wrapper kwargs: {kwargs}")
         high, low, close =df_ohlc['high'].values, df_ohlc['low'].values, df_ohlc['close'].values
         kwargs["high"] = high
         kwargs["low"] = low
         kwargs["close"] = close
         func_result = func(*args, **kwargs)
-        print(f"func_result shape: {len(func_result)}")
-        trans_start = time.time()
-        result_kwargs = pd.DataFrame(func_result).transpose()
-        trans_end = time.time()
-        print(f"transpose duration: {(trans_end - trans_start) * 1000} ms")
+        if isinstance(func_result, tuple):
+            result_kwargs = pd.DataFrame(func_result).transpose()
+        else:
+            result_kwargs = pd.DataFrame(func_result)
         # concat_start - time.time()
         # for i in range(len(func_result))
         # print(f"result decorator: \n{result_kwargs}")
@@ -23,6 +22,27 @@ def pandas_decorator(func):
     return wrapper
 
 
+@pandas_decorator
+def atr_pd(high, low, close, length):
+    return atr(high, low, close, length)
+
+
+@pandas_decorator
+def dmi_pd(high, low, close, di_length, adx_length):
+    return dmi(high, low, close, di_length, adx_length)
+
+
+@pandas_decorator
+def bb_pd(source, length, mult):
+    nb_source = source.values
+    return bb_pd(nb_source, length, mult)
+
+
+@pandas_decorator
+def ds_pd(high, low, close, factor, atr_length):
+    return ds(high, low, close, factor, atr_length)
+
+
 @nb.jit(
     (nb.float64[:], nb.float64[:], nb.float64[:], nb.int16),
     nopython=True, nogil=True, cache=True
@@ -52,36 +72,6 @@ def atr(high, low, close, length):
 
     return values
 
-@pandas_decorator
-@nb.jit(
-    (nb.float64[:], nb.float64[:], nb.float64[:], nb.int16),
-    nopython=True, nogil=True, cache=True
-)
-def atr_pd(high, low, close, length):
-    # tr
-    hl = high - low
-    hc = np.absolute(
-        high - np.concatenate((np.full(1, np.nan), close[:-1]))
-    )
-    lc = np.absolute(
-        low - np.concatenate((np.full(1, np.nan), close[:-1]))
-    )
-    hc[0] = abs(high[0] - close[0])
-    lc[0] = abs(low[0] - close[0])
-    tr = np.maximum(np.maximum(hl, hc), lc)
-
-    # values
-    alpha = 1 / length
-    values = tr.copy()
-    na_sum = np.isnan(values).sum()
-    values[length + na_sum - 1] = tr[na_sum : length + na_sum].mean()
-    values[: length + na_sum - 1] = np.nan
-
-    for i in range(length + na_sum, values.shape[0]):
-        values[i] = alpha * values[i] + (1 - alpha) * values[i - 1]
-
-    return values
-
 
 @nb.jit(
     (nb.float64[:], nb.int16, nb.float32), 
@@ -381,64 +371,6 @@ def donchian(source1, source2, length):
     return values
 
 
-@pandas_decorator
-@nb.jit(
-    (nb.float64[:], nb.float64[:], nb.float64[:], nb.float32, nb.int16),
-    nopython=True, nogil=True, cache=True
-)
-def ds_pd(high, low, close, factor, atr_length):
-    # tr
-    hl = high - low
-    hc = np.absolute(
-        high - np.concatenate((np.full(1, np.nan), close[:-1]))
-    )
-    lc = np.absolute(
-        low - np.concatenate((np.full(1, np.nan), close[:-1]))
-    )
-    hc[0] = abs(high[0] - close[0])
-    lc[0] = abs(low[0] - close[0])
-    tr = np.maximum(np.maximum(hl, hc), lc)
-
-    # atr
-    atr = tr.copy()
-    alpha = 1 / atr_length
-    na_sum = np.isnan(atr).sum()
-    atr[: atr_length + na_sum - 1] = np.nan
-    atr[atr_length + na_sum - 1] = tr[na_sum : atr_length + na_sum].mean()
-
-    for i in range(atr_length + na_sum, atr.shape[0]):
-        atr[i] = alpha * atr[i] + (1 - alpha) * atr[i - 1]
-
-    # values
-    hl2 = (high + low) / 2
-    upper_band = hl2 + factor * atr
-    lower_band = hl2 - factor * atr
-
-    for i in range(1, atr.shape[0]):
-        if not np.isnan(upper_band[i - 1]):
-            prev_upper_band = upper_band[i - 1]
-        else:
-            prev_upper_band = 0
-
-        if not np.isnan(lower_band[i - 1]):
-            prev_lower_band = lower_band[i - 1]
-        else:
-            prev_lower_band = 0
-
-        if not (upper_band[i] < prev_upper_band
-                or close[i - 1] > prev_upper_band):
-            upper_band[i] = prev_upper_band
-
-        if not (lower_band[i] > prev_lower_band
-                or close[i - 1] < prev_lower_band):
-            lower_band[i] = prev_lower_band
-
-    upper_band[upper_band == 0] = np.nan
-    lower_band[lower_band == 0] = np.nan
-    values = (upper_band, lower_band)
-    return values
-
-
 @nb.jit(
     (nb.float64[:], nb.float64[:], nb.float64[:], nb.float32, nb.int16),
     nopython=True, nogil=True, cache=True
